/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.factory.client.render;

import javax.annotation.Nonnull;

import net.minecraft.client.renderer.VertexBuffer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;

import net.minecraftforge.client.model.animation.FastTESR;

import buildcraft.lib.client.render.laser.LaserData_BC8;
import buildcraft.lib.client.render.laser.LaserData_BC8.LaserType;
import buildcraft.lib.client.render.laser.LaserRenderer_BC8;

import buildcraft.factory.tile.TileMiner;

public class RenderTube extends FastTESR<TileMiner> {
    private final LaserType laserType;

    public RenderTube(LaserType laserType) {
        this.laserType = laserType;
    }

    @Override
    public void renderTileEntityFast(@Nonnull TileMiner tile, double x, double y, double z, float partialTicks, int destroyStage, @Nonnull VertexBuffer buffer) {
        double tubeY = tile.func_174877_v().func_177956_o() - tile.getLength(partialTicks);
        if (tubeY == 0) {
            return;
        }

        BlockPos from = tile.func_174877_v();
        buffer.func_178969_c(x - from.func_177958_n(), y - from.func_177956_o(), z - from.func_177952_p());

        Vec3d start = new Vec3d(from.func_177958_n() + 0.5, from.func_177956_o(), from.func_177952_p() + 0.5);

        Vec3d end = new Vec3d(from.func_177958_n() + 0.5, tubeY, from.func_177952_p() + 0.5);

        LaserData_BC8 data = new LaserData_BC8(laserType, start, end, 1 / 16.0);
        LaserRenderer_BC8.renderLaserDynamic(data, buffer);

        buffer.func_178969_c(0, 0, 0);
    }
}
