/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.factory.tile;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;
import net.minecraft.world.WorldServer;

import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.world.BlockEvent;

import buildcraft.api.core.BuildCraftAPI;
import buildcraft.api.core.EnumPipePart;
import buildcraft.api.mj.IMjReceiver;

import buildcraft.lib.inventory.AutomaticProvidingTransactor;
import buildcraft.lib.misc.BlockUtil;
import buildcraft.lib.misc.CapUtil;
import buildcraft.lib.misc.InventoryUtil;
import buildcraft.lib.mj.MjBatteryReceiver;

import buildcraft.factory.BCFactoryBlocks;

public class TileMiningWell extends TileMiner {
    public TileMiningWell() {
        super();
        caps.addCapabilityInstance(CapUtil.CAP_ITEM_TRANSACTOR, AutomaticProvidingTransactor.INSTANCE, EnumPipePart.VALUES);
    }

    @Override
    protected void mine() {
        if (currentPos != null && canBreak()) {
            long target = BlockUtil.computeBlockBreakPower(field_145850_b, currentPos);
            progress += battery.extractPower(0, target - progress);
            if (progress >= target) {
                progress = 0;
                EntityPlayer fakePlayer = BuildCraftAPI.fakePlayerProvider.getFakePlayer((WorldServer) field_145850_b, getOwner(), field_174879_c);
                BlockEvent.BreakEvent breakEvent = new BlockEvent.BreakEvent(field_145850_b, currentPos, field_145850_b.func_180495_p(currentPos), fakePlayer);
                MinecraftForge.EVENT_BUS.post(breakEvent);
                if (!breakEvent.isCanceled()) {
                    NonNullList<ItemStack> stacks = BlockUtil.getItemStackFromBlock((WorldServer) field_145850_b, currentPos, getOwner());
                    if (stacks != null) {
                        for (ItemStack stack : stacks) {
                            InventoryUtil.addToBestAcceptor(field_145850_b, field_174879_c, null, stack);
                        }
                    }
                    field_145850_b.func_175715_c(currentPos.hashCode(), currentPos, -1);
                    field_145850_b.func_175655_b(currentPos, false);
                }
                nextPos();
                updateLength();
            } else {
                if (!field_145850_b.func_175623_d(currentPos)) {
                    field_145850_b.func_175715_c(currentPos.hashCode(), currentPos, (int) ((progress * 9) / target));
                }
            }
        } else {
            nextPos();
            updateLength();
        }
    }

    private boolean canBreak() {
        return !field_145850_b.func_175623_d(currentPos) && !BlockUtil.isUnbreakableBlock(field_145850_b, currentPos, getOwner());
    }

    private void nextPos() {
        for (currentPos = field_174879_c.func_177977_b(); currentPos.func_177956_o() >= 0; currentPos = currentPos.func_177977_b()) {
            if (canBreak()) {
                updateLength();
                return;
            } else if (!field_145850_b.func_175623_d(currentPos) && field_145850_b.func_180495_p(currentPos).func_177230_c() != BCFactoryBlocks.tube) {
                break;
            }
        }
        updateLength();
        currentPos = null;
    }

    @Override
    protected void initCurrentPos() {
        if (currentPos == null) {
            nextPos();
        }
    }

    @Override
    public void func_145843_s() {
        if (currentPos != null) {
            field_145850_b.func_175715_c(currentPos.hashCode(), currentPos, -1);
        }
        super.func_145843_s();
    }

    @Override
    protected IMjReceiver createMjReceiver() {
        return new MjBatteryReceiver(battery);
    }
}
