/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.engine;

import java.util.EnumMap;
import java.util.Locale;
import java.util.Map;
import java.util.function.Supplier;

import javax.annotation.Nonnull;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumBlockRenderType;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import buildcraft.api.blocks.ICustomRotationHandler;
import buildcraft.api.core.IEngineType;

import buildcraft.lib.block.BlockBCTile_Neptune;
import buildcraft.lib.misc.EntityUtil;
import buildcraft.lib.registry.RegistryHelper;

public abstract class BlockEngineBase_BC8<E extends Enum<E> & IEngineType> extends BlockBCTile_Neptune implements ICustomRotationHandler {
    private final Map<E, Supplier<? extends TileEngineBase_BC8>> engineTileConstructors = new EnumMap<>(getEngineProperty().func_177699_b());

    public BlockEngineBase_BC8(Material material, String id) {
        super(material, id);
    }

    // Engine directly related methods

    public void registerEngine(E type, Supplier<? extends TileEngineBase_BC8> constructor) {
        if (RegistryHelper.isEnabled("engines", getRegistryName() + "/" + type.name().toLowerCase(Locale.ROOT), getUnlocalizedName(type))) {
            engineTileConstructors.put(type, constructor);
        }
    }

    public boolean isRegistered(E type) {
        return engineTileConstructors.containsKey(type);
    }

    @Nonnull
    public ItemStack getStack(E type) {
        return new ItemStack(this, 1, type.ordinal());
    }

    public abstract IProperty<E> getEngineProperty();

    public abstract E getEngineType(int meta);

    public abstract String getUnlocalizedName(E engine);

    // BlockState

    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, getEngineProperty());
    }

    @Override
    public int func_176201_c(IBlockState state) {
        E type = state.func_177229_b(getEngineProperty());
        return type.ordinal();
    }

    @Override
    public IBlockState func_176203_a(int meta) {
        E engineType = getEngineType(meta);
        return func_176223_P().func_177226_a(getEngineProperty(), engineType);
    }

    // Misc Block Overrides

    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Override
    public boolean func_149730_j(IBlockState state) {
        return false;
    }

    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @Override
    public boolean isSideSolid(IBlockState base_state, IBlockAccess world, BlockPos pos, EnumFacing side) {
        TileEntity tile = world.func_175625_s(pos);
        if (tile instanceof TileEngineBase_BC8) {
            TileEngineBase_BC8 engine = (TileEngineBase_BC8) tile;
            return side == engine.currentDirection.func_176734_d();
        }
        return false;
    }

    @Override
    public EnumBlockRenderType func_149645_b(IBlockState state) {
        return EnumBlockRenderType.ENTITYBLOCK_ANIMATED;
    }

    @Override
    public TileEntity func_149915_a(World world, int meta) {
        IBlockState state = func_176203_a(meta);
        E engineType = state.func_177229_b(getEngineProperty());
        Supplier<? extends TileEngineBase_BC8> constructor = engineTileConstructors.get(engineType);
        if (constructor == null) {
            return null;
        }
        TileEngineBase_BC8 tile = constructor.get();
        tile.func_145834_a(world);
        return tile;
    }

    @Override
    public void func_149666_a(Item item, CreativeTabs tab, NonNullList<ItemStack> list) {
        for (E engine : getEngineProperty().func_177700_c()) {
            if (engineTileConstructors.containsKey(engine)) {
                list.add(new ItemStack(item, 1, engine.ordinal()));
            }
        }
    }

    @Override
    public boolean func_180639_a(World world, BlockPos pos, IBlockState state, EntityPlayer player, EnumHand hand, EnumFacing side, float hitX, float hitY, float hitZ) {
        if (EntityUtil.getWrenchHand(player) != null && !player.func_70093_af()) {
            return false;
        }
        TileEntity tile = world.func_175625_s(pos);
        if (tile instanceof TileEngineBase_BC8) {
            TileEngineBase_BC8 engine = (TileEngineBase_BC8) tile;
            return engine.onActivated(player, hand, side, hitX, hitY, hitZ);
        }
        return false;
    }

    @Override
    public int func_180651_a(IBlockState state) {
        return state.func_177229_b(getEngineProperty()).ordinal();
    }

    @Override
    public void func_189540_a(IBlockState state, World world, BlockPos pos, Block blockIn, BlockPos fromPos) {
        if (world.field_72995_K) return;
        TileEntity tile = world.func_175625_s(pos);
        if (tile instanceof TileEngineBase_BC8) {
            TileEngineBase_BC8 engine = (TileEngineBase_BC8) tile;
            engine.rotateIfInvalid();
        }
    }

    // ICustomRotationHandler

    @Override
    public EnumActionResult attemptRotation(World world, BlockPos pos, IBlockState state, EnumFacing sideWrenched) {
        TileEntity tile = world.func_175625_s(pos);
        if (tile instanceof TileEngineBase_BC8) {
            TileEngineBase_BC8 engine = (TileEngineBase_BC8) tile;
            return engine.attemptRotation();
        }
        return EnumActionResult.FAIL;
    }
}
